/**
 * Created by IntelliJ IDEA.
 * User: moyez
 * Date: Mar 13, 2008
 * Time: 7:30:37 PM
 * To change this template use File | Settings | File Templates.
 */

package org.baderlab.brain;

public class Brain {

    //logo tree leaf node trimming factor
    public final static double LEAFNODE_TRIM_FACTOR_DEFAULT = 0.100;
    public final static double LEAFNODE_TRIM_FACTOR_MIN = 0;
    public final static double LEAFNODE_TRIM_FACTOR_MAX = 1.000;

    //logo tree internal node trimming factor
    public final static double INTERNALNODE_TRIM_FACTOR_DEFAULT = 0.100;
    public final static double INTERNALNODE_TRIM_FACTOR_MIN = 0;
    public final static double INTERNALNODE_TRIM_FACTOR_MAX = 1.000;

    
    //logo tree leaf ordering
    public final static String LEAF_ORDERING_BARJOSEPH2003 = "Bar-Joseph";
    public final static String LEAF_ORDERING_EISEN = "Eisen Heuristic";
    public final static String LEAF_ORDERING_DEFAULT = LEAF_ORDERING_BARJOSEPH2003;
    public final static String[] LEAF_ORDERING_METHODS = {
            LEAF_ORDERING_BARJOSEPH2003,
            LEAF_ORDERING_EISEN
    };

}
